"""
Base file for all classes.
"""


class Encoder:
    """
    Base for encoders.
    """

    def __init__(self):
        pass

    def get_spikes(self):
        pass

    def get_spike_time(self):
        pass

    def save(self):
        pass
